@@ -17,10 +17,7 @@
 #include "settings.h"
 #include "unknownrecipientwidget.h"
 
-#include "commands/detailscommand.h"
-
 #include "dialogs/certificateselectiondialog.h"
-#include "dialogs/groupdetailsdialog.h"
 
 #include <QVBoxLayout>
 #include <QHBoxLayout>
@@ -259,8 +256,10 @@ void SignEncryptWidget::setEncryptWithPasswordText(const QString& text)
 
 CertificateLineEdit *SignEncryptWidget::addRecipientWidget()
 {
-    auto certSel = new CertificateLineEdit(mModel, this,
-                                           new EncryptCertificateFilter(mCurrentProto));
+    auto certSel = new CertificateLineEdit(mModel,
+                                           new EncryptCertificateFilter(mCurrentProto),
+                                           this);
+    certSel->setAccessibleNameOfLineEdit(i18nc("text for screen readers", "recipient key"));
     certSel->setEnabled(mEncOtherChk->isChecked());
     mRecpWidgets << certSel;
 
@@ -276,8 +275,8 @@ CertificateLineEdit *SignEncryptWidget::addRecipientWidget()
             this, &SignEncryptWidget::recpRemovalRequested);
     connect(certSel, &CertificateLineEdit::editingStarted,
             this, &SignEncryptWidget::recipientsChanged);
-    connect(certSel, &CertificateLineEdit::dialogRequested,
-            this, [this, certSel] () { dialogRequested(certSel); });
+    connect(certSel, &CertificateLineEdit::certificateSelectionRequested,
+            this, [this, certSel]() { certificateSelectionRequested(certSel); });
 
     return certSel;
 }
@@ -300,32 +299,30 @@ void SignEncryptWidget::addRecipient(const KeyGroup &group)
     }
 }
 
-void SignEncryptWidget::dialogRequested(CertificateLineEdit *certificateLineEdit)
+void SignEncryptWidget::certificateSelectionRequested(CertificateLineEdit *certificateLineEdit)
 {
-    if (!certificateLineEdit->key().isNull()) {
-        auto cmd = new Commands::DetailsCommand(certificateLineEdit->key(), nullptr);
-        cmd->start();
-        return;
-    }
-    if (!certificateLineEdit->group().isNull()) {
-        auto dlg = new GroupDetailsDialog;
-        dlg->setAttribute(Qt::WA_DeleteOnClose);
-        dlg->setGroup(certificateLineEdit->group());
-        dlg->show();
-        return;
-    }
+    CertificateSelectionDialog dlg{this};
 
-    auto const dlg = new CertificateSelectionDialog(this);
-
-    dlg->setOptions(CertificateSelectionDialog::Options(
+    dlg.setOptions(CertificateSelectionDialog::Options(
         CertificateSelectionDialog::MultiSelection |
         CertificateSelectionDialog::EncryptOnly |
         CertificateSelectionDialog::optionsFromProtocol(mCurrentProto) |
         CertificateSelectionDialog::IncludeGroups));
 
-    if (dlg->exec()) {
-        const std::vector<Key> keys = dlg->selectedCertificates();
-        const std::vector<KeyGroup> groups = dlg->selectedGroups();
+    if (!certificateLineEdit->key().isNull()) {
+        const auto key = certificateLineEdit->key();
+        const auto name = QString::fromUtf8(key.userID(0).name());
+        const auto email = QString::fromUtf8(key.userID(0).email());
+        dlg.setStringFilter(!name.isEmpty() ? name : email);
+    } else if (!certificateLineEdit->group().isNull()) {
+        dlg.setStringFilter(certificateLineEdit->group().name());
+    } else {
+        dlg.setStringFilter(certificateLineEdit->text());
+    }
+
+    if (dlg.exec()) {
+        const std::vector<Key> keys = dlg.selectedCertificates();
+        const std::vector<KeyGroup> groups = dlg.selectedGroups();
         if (keys.size() == 0 && groups.size() == 0) {
             return;
         }
@@ -347,7 +344,7 @@ void SignEncryptWidget::dialogRequested(CertificateLineEdit *certificateLineEdit
             }
         }
     }
-    delete dlg;
+
     recipientsChanged();
 }
 
@@ -633,9 +630,13 @@ void Kleo::SignEncryptWidget::onProtocolChanged()
 
 bool SignEncryptWidget::validate()
 {
+    CertificateLineEdit *firstUnresolvedRecipient = nullptr;
     QStringList unresolvedRecipients;
     for (const auto edit: std::as_const(mRecpWidgets)) {
         if (edit->isEnabled() && !edit->isEmpty() && edit->key().isNull() && edit->group().isNull()) {
+            if (!firstUnresolvedRecipient) {
+                firstUnresolvedRecipient = edit;
+            }
             unresolvedRecipients.push_back(edit->text().toHtmlEscaped());
         }
     }
@@ -645,6 +646,9 @@ bool SignEncryptWidget::validate()
                                unresolvedRecipients,
                                i18n("Failed to find some keys"));
     }
+    if (firstUnresolvedRecipient) {
+        firstUnresolvedRecipient->setFocus();
+    }
     return unresolvedRecipients.isEmpty();
 }
 
