@@ -7,8 +7,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_CRYPTO_TASK_H__
-#define __KLEOPATRA_CRYPTO_TASK_H__
+#pragma once
 
 #include <QObject>
 #include <QString>
@@ -35,7 +34,7 @@ class Task : public QObject
     Q_OBJECT
 public:
     explicit Task(QObject *parent = nullptr);
-    ~Task();
+    ~Task() override;
 
     class Result;
 
@@ -129,5 +128,4 @@ private:
 }
 }
 
-#endif /* __KLEOPATRA_CRYPTO_TASK_H__ */
 
