@@ -18,6 +18,7 @@
 #include "kleopatra_options.h"
 #include "systrayicon.h"
 #include "settings.h"
+#include "smimevalidationpreferences.h"
 
 #include <smartcard/readerstatus.h>
 #include <conf/configuredialog.h>
@@ -28,8 +29,11 @@
 
 #include <gpgme++/key.h>
 
+#include <Libkleo/Dn>
 #include <Libkleo/FileSystemWatcher>
 #include <Libkleo/KeyCache>
+#include <Libkleo/KeyFilterManager>
+#include <Libkleo/KeyGroupConfig>
 #include <Libkleo/Classify>
 
 #ifdef HAVE_USABLE_ASSUAN
@@ -52,6 +56,7 @@
 #include <KMessageBox>
 #include <KWindowSystem>
 
+#include <QDesktopServices>
 #include <QFile>
 #include <QDir>
 #include <QPointer>
@@ -86,10 +91,11 @@ class KleopatraApplication::Private
     KleopatraApplication *const q;
 public:
     explicit Private(KleopatraApplication *qq)
-        : q(qq),
-          ignoreNewInstance(true),
-          firstNewInstance(true),
-          sysTray(nullptr)
+        : q(qq)
+        , ignoreNewInstance(true)
+        , firstNewInstance(true)
+        , sysTray(nullptr)
+        , groupConfig{std::make_shared<KeyGroupConfig>(QStringLiteral("kleopatragroupsrc"))}
     {
     }
     ~Private() {
@@ -97,7 +103,7 @@ public:
         delete sysTray;
 #endif
     }
-    void init()
+    void setUpSysTrayIcon()
     {
         KDAB_SET_OBJECT_NAME(readerStatus);
 #ifndef QT_NO_SYSTEMTRAYICON
@@ -145,6 +151,7 @@ public:
 #ifndef QT_NO_SYSTEMTRAYICON
     SysTrayIcon *sysTray;
 #endif
+    std::shared_ptr<KeyGroupConfig> groupConfig;
     std::shared_ptr<KeyCache> keyCache;
     std::shared_ptr<Log> log;
     std::shared_ptr<FileSystemWatcher> watcher;
@@ -153,16 +160,24 @@ public:
     void setupKeyCache()
     {
         keyCache = KeyCache::mutableInstance();
+        keyCache->setRefreshInterval(SMimeValidationPreferences{}.refreshInterval());
         watcher.reset(new FileSystemWatcher);
 
         watcher->whitelistFiles(gnupgFileWhitelist());
         watcher->addPath(gnupgHomeDirectory());
         watcher->setDelay(1000);
         keyCache->addFileSystemWatcher(watcher);
-        keyCache->setGroupsConfig(QStringLiteral("kleopatragroupsrc"));
+        keyCache->setGroupConfig(groupConfig);
         keyCache->setGroupsEnabled(Settings().groupsEnabled());
     }
 
+    void setUpFilterManager()
+    {
+        if (!Settings{}.cmsEnabled()) {
+            KeyFilterManager::instance()->alwaysFilterByProtocol(GpgME::OpenPGP);
+        }
+    }
+
     void setupLogging()
     {
         log = Log::mutableInstance();
@@ -210,9 +225,17 @@ void KleopatraApplication::init()
     QWindowsWindowFunctions::setWindowActivationBehavior(
             QWindowsWindowFunctions::AlwaysActivateWindow);
 #endif
-    d->init();
+    const auto blockedUrlSchemes = Settings{}.blockedUrlSchemes();
+    for (const auto &scheme : blockedUrlSchemes) {
+        QDesktopServices::setUrlHandler(scheme, this, "blockUrl");
+    }
     add_resources();
+    DN::setAttributeOrder(Settings{}.attributeOrder());
+    connect(&d->readerStatus, &SmartCard::ReaderStatus::startOfGpgAgentRequested,
+            this, &KleopatraApplication::startGpgAgent);
     d->setupKeyCache();
+    d->setUpSysTrayIcon();
+    d->setUpFilterManager();
     d->setupLogging();
 #ifndef QT_NO_SYSTEMTRAYICON
     d->sysTray->show();
@@ -668,3 +691,15 @@ bool KleopatraApplication::ignoreNewInstance() const
 {
     return d->ignoreNewInstance;
 }
+
+void KleopatraApplication::blockUrl(const QUrl &url)
+{
+    qCDebug(KLEOPATRA_LOG) << "Blocking URL" << url;
+    KMessageBox::sorry(mainWindow(),i18n ("Opening an external link is administratively prohibited."),
+                i18n ("Prohibited"));
+}
+
+void KleopatraApplication::startGpgAgent()
+{
+    Kleo::launchGpgAgent();
+}
